@@ -3,13 +3,14 @@
 '''
 A library to access Tryton's models like a client.
 '''
-__version__ = "1.8.5"
+__version__ = "2.0.1"
 __all__ = ['Model', 'Wizard']
 from types import NoneType
 import threading
 import datetime
 from decimal import Decimal
 import proteus.config
+from proteus.pyson import PYSONDecoder
 
 _MODELS = threading.local()
 
@@ -161,8 +162,7 @@ class Many2OneDescriptor(FieldDescriptor):
             value = relation(value)
         elif not value:
             value = None
-        if self.name in instance._values:
-            instance._values[self.name] = value
+        instance._values[self.name] = value
         return value
 
     def __set__(self, instance, value):
@@ -497,9 +497,6 @@ class Model(object):
     @classmethod
     def get(cls, name, config=None):
         'Get a class for the named Model'
-        if isinstance(name, unicode):
-            name = name.encode('utf-8')
-
         class Spam(Model):
             __metaclass__ = MetaModelFactory(name, config=config)()
         return Spam
@@ -559,7 +556,7 @@ class Model(object):
         'Save the record'
         context = self._config.context
         if self.id < 0:
-            values = self._get_values()
+            values = self._get_values(fields=self._changed)
             self.__id = self._proxy.create(values, context)
         else:
             if not self._changed:
@@ -670,8 +667,11 @@ class Model(object):
         return res
 
     def _on_change_set(self, field, value):
-        if (self._fields[field]['type'] in ('one2many', 'many2many')
-                and not isinstance(value, (list, tuple))):
+        if self._fields[field]['type'] in ('one2many', 'many2many'):
+            if isinstance(value, (list, tuple)):
+                self._values[field] = value
+                self._changed.add(field)
+                return
             to_remove = []
             if value and value.get('remove'):
                 for record_id in value['remove']:
@@ -685,8 +685,12 @@ class Model(object):
                 for vals in value.get('add', []):
                     relation = Model.get(self._fields[field]['relation'],
                             self._config)
+                    record = relation()
+                    for i, j in vals.iteritems():
+                        record._values[i] = j
+                        record._changed.add(i)
                     # append without signal
-                    list.append(getattr(self, field), relation(**vals))
+                    list.append(getattr(self, field), record)
                 for vals in value.get('update', []):
                     if 'id' not in vals:
                         continue
@@ -697,12 +701,10 @@ class Model(object):
                                 record._changed.add(i)
         else:
             self._values[field] = value
-        self._changed.add(field)
+            self._changed.add(field)
 
     def _on_change(self, name):
         'Call on_change for field'
-        # Import locally to not break installation
-        from proteus.pyson import PYSONDecoder
         definition = self._fields[name]
         if definition.get('on_change'):
             if isinstance(definition['on_change'], basestring):
@@ -721,13 +723,6 @@ class Model(object):
                 self._on_change_set(field, value)
             for field, value in later.iteritems():
                 self._on_change_set(field, value)
-            if self._parent:
-                self._parent._changed.add(self._parent_field_name)
-        if definition.get('change_default'):
-            context = self._config.context
-            default = Model.get('ir.default', config=self._config)
-            self._default_set(default.get_default(self.__class__.__name__,
-                '%s=%s' % (name, self._get_values([name])[name]), context))
         for field, definition in self._fields.iteritems():
             if not definition.get('on_change_with'):
                 continue
@@ -740,6 +735,9 @@ class Model(object):
             res = getattr(self._proxy, 'on_change_with_%s' % field)(args,
                     context)
             self._on_change_set(field, res)
+        if self._parent:
+            self._parent._changed.add(self._parent_field_name)
+            self._parent._on_change(self._parent_field_name)
 
 
 class Wizard(object):
